<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\symfony_webhook_receiver\DependencyInjection;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Parameter;
use Symfony\Component\DependencyInjection\Reference;

/**
 * Compiler pass responsible for registering Webhook request parsers.
 *
 * @internal This class is not part of the module's public programming API.
 */
final class RegisterWebhookRequestParsersCompilerPass implements CompilerPassInterface {

  /**
   * {@inheritdoc}
   */
  public function process(ContainerBuilder $container): void {
    /** @var array<string, array{parser: \Symfony\Component\Webhook\Client\RequestParserInterface, secret: string}> $parsers */
    $parsers = [];
    foreach ($container->findTaggedServiceIds('webhook.request_parser') as $id => $tags) {
      $request_parser_type = $tags[0]['type'];
      assert(!empty($request_parser_type), sprintf('The "type" attribute is set on the "%s" webhook request parser. ', $id));
      $secret_container_parameter_name = "symfony_webhook_receiver.request_parser.{$request_parser_type}.secret";
      if (!$container->hasParameter($secret_container_parameter_name)) {
        throw new \LogicException(sprintf("The secret parameter '%s' is required.", $secret_container_parameter_name));
      }
      $parsers[$request_parser_type] = [
        'parser' => new Reference($id),
        'secret' => new Parameter($secret_container_parameter_name),
      ];
    }

    $webhook_controller = $container->getDefinition('symfony_webhook_receiver.webhook_controller');
    $webhook_controller->setArgument('$parsers', $parsers);
  }

}
