<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */
namespace Drupal\Tests\symfony_webhook_receiver\Kernel;

use ColinODell\PsrTestLogger\TestLogger;
use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\HttpFoundation\Request;

/**
 * Tests the webhook receiver component's integration with Drupal.
 *
 * @internal This class is not part of the module's public programming API.
 *
 * @todo Cover unhappy paths (like HTTP 406) with tests.
 */
final class WebhookReceiverIntegrationTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['sm', 'symfony_webhook_receiver', 'symfony_webhook_receiver_example'];

  /**
   * The test logger.
   *
   * @var \ColinODell\PsrTestLogger\TestLogger
   */
  private TestLogger $testLogger;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->testLogger = new TestLogger();
    $this->container->set('logger.channel.symfony_webhook_receiver_example.example_consumer', $this->testLogger);
    $this->container->set('logger.channel.symfony_webhook_receiver_example.example_auto_discover_consumer', $this->testLogger);
  }

  /**
   * Tests webhook receiver handles known webhook type.
   *
   * @dataProvider webhookTypesFromExampleModule
   */
  public function testWebhookReceiverHandlesKnownWebhookType(string $webhook_type): void {
    // @phpstan-ignore argument.type
    $request = Request::create('/webhook/' . $webhook_type, 'POST', content: json_encode((object) [
      'empty' => 'body',
    ]));

    $request->headers->set('Webhook-Signature', 'sha256=aa96b0f62e7b7708dd8aae7d01379a5a767374d30adc083c653e9efb0aef2a3a');
    $request->headers->set('Webhook-Event', 'test-event');
    $request->headers->set('Webhook-Id', 'test-id');

    $response = \Drupal::service('http_kernel')->handle($request);
    $response->prepare($request);

    $this->assertEquals(202, $response->getStatusCode());
    self::assertTrue($this->testLogger->hasDebugThatContains(print_r([
      'name' => 'test-event',
      'id' => 'test-id',
      'payload' => ['empty' => 'body'],
    ], TRUE)));
  }

  /**
   * Provides webhook types defined in symfony_webhook_receiver_example.
   *
   * @return \Generator<string,string[]>
   *   Webhook types.
   */
  public static function webhookTypesFromExampleModule(): \Generator {
    yield 'example' => ['example'];
    yield 'example_auto_discover' => ['example_auto_discover'];
  }

}
